// C Header File
// Created 7/7/00; 9:52:34 PM

/*-----------------------Painter 89--------------------------------------
This header file contains the guts of the program. All of the functions
that I wrote are in here. 
------------------------------------------------------------------------*/

//color defines
#define ERASE 0
#define LGREY 1
#define DGREY 2
#define BLACK 3

//mode defines
#define NONE 0 //no drawing
#define DRAW 1 //pen drawing
#define CIRCLE 2 //circle drawing
#define LINE 3 //line drawing

//global plane variables  

void *plane0;
void *plane1;
void *vplane0;
void *vplane1;

int max_y;

//compress data (this is a bit of inline assembly)
void compress (void *src, void *dest, int len)
{
  asm ("move.l %0, %%a0" :: "g"(src));
  asm ("move.l %0, %%a1" :: "g"(dest));
  asm ("move.w %0, %%d0" :: "g"(len));
  asm ("jsr ziplib__0004");
}

//decompress data (this is a bit of inline assembly)
void extract (void *src, void *dest)
{
  asm ("move.l %0, %%a0" :: "g"(src));
  asm ("move.l %0, %%a1" :: "g"(dest));
  asm ("jsr ziplib__0005");
}

/*This replaces the fwrite function from stdio.h, because it's buggy.
Thanks to Zeljko Juric and everyone on A89 for helping me with this*/

#undef fwrite // this will cancel old definition from stdio.h
unsigned fwrite(void *ptr,unsigned size,unsigned n,FILE *f)
{
  unsigned i,j;
  int saveflags=f->flags;
  f->flags|=_F_BIN;
  for(i=0;i<n;i++)
    for(j=0;j<size;j++)
      if(fputc(*(unsigned char*)ptr++,f)<0) goto exit;
exit:
  f->flags=saveflags;
  return i;
}



//Clears Greyscale screen (both virtual and real)
void GreyClrScr (void)
{
	PortSet (plane0,239,127);
	ClrScr ();
	PortSet (plane1,239,127);
	ClrScr ();
	PortSet (vplane0,239,127);
	ClrScr ();
	PortSet (vplane1,239,127);
	ClrScr ();
}

//bails the program if a memory allocation fails
void MemError (void *memory)
{
	if (!memory)
	{
		GreyClrScr ();
		DrawStr (10,10,"Not Enough Memory",A_NORMAL);
		ngetchx ();
		abort ();
	}
}

//game initialization
void GameInit (void)
{
	//turns greyscale on
	GrayMode (GRAY_ON);
	
	//allocates memory for the virtual screen
	vplane0 = malloc (LCD_SIZE);
	MemError (vplane0);
	
	vplane1 = malloc (LCD_SIZE);
	MemError (vplane1);
	
	//gets the address of the real grey bitplanes
	plane0 = GetPlane (0);
	plane1 = GetPlane (1);
	
	//clears the screen
	GreyClrScr ();
}

//cleans up everything for shutdown
void GameShutDown (void)
{
	//restores drawing port, so TIOS isn't fooled
	PortRestore ();
	
	//frees the memory allocated to the virtual screens (IMPORTANT)
	free (vplane0);
	free (vplane1);
	
	//turns greyscale off
	GrayMode (GRAY_OFF);		
	ClrScr ();
}

//copies virtual screen to main screen
void PageFlip (void)
{
	memcpy (plane0,vplane0,LCD_SIZE);
	memcpy (plane1,vplane1,LCD_SIZE);		
}

//input a string
void InputStr (char *buffer, int maxlen)
{
  SCR_STATE ss;
  int key, i = 0;
  buffer[0] = 0;
  SaveScrState (&ss);
  do {
   MoveTo (5,20);
   printf ("%s_   ", buffer);  // Two spaces are required if F_4x6 font is used
   key = ngetchx ( );
   if (key >= ' ' && key <= '~' && i < maxlen) buffer[i++] = key;
   if (key == 257 && i) i--;
   buffer[i] = 0;
  } while (key != 13);
}

//saves the screen to a variable of type PNT
int SavePic (void)
{
	FILE *fp; 			//pointer to file structure
	void *temp;			//holds compressed data	
	char file[9];		//filename
	
	//allocate some memory to hold compressed data
	temp = malloc(1500);
	MemError (temp);
			
	//asks for a filename
	FontSetSys (F_6x8);
	PortSet (plane0,239,127);
	ClrScr ();
	PortSet (plane1,239,127);
	ClrScr ();
	DrawStr (44,0,"Save Picture",A_NORMAL);
	InputStr (file,8);
	
	//opens the file, and checks for errors
	fp = fopen (file,"wb");
	if (fp==NULL)
	{
		DrawStr (5,40,"Error",A_NORMAL);
		DrawStr (80,40,"Hit any key",A_NORMAL);
		ngetchx ();
		return 1;
	}
	//compresses the screen, and writes it to the file
	compress (vplane0,temp,3840);		
	fwrite (temp,1500,1,fp);
	compress (vplane1,temp,3840);
	fwrite (temp,1500,1,fp);
	
	//give the file type PNT
	fputc (0,fp);
	fputs ("PNT",fp);
	fputc (0,fp);
	fputc (OTH_TAG,fp);
	fclose (fp);
	free (temp);
	
	DrawStr (5,40,"File Saved!",A_NORMAL);
	DrawStr (80,40,"Hit any key",A_NORMAL);
	ngetchx ();
	return 0;
}

//loads a saved screen
int LoadPic (void)
{
	FILE *fp;				//pointer to file structure
	void *temp;			//holds compressed data
	char file[9];		//filename
	
	//allocate memory for compressed data
	temp = malloc(1500);
	MemError (temp);
	
	//asks for a filename
	FontSetSys (F_6x8);
	PortSet (plane0,239,127);
	ClrScr ();
	PortSet (plane1,239,127);
	ClrScr ();
	DrawStr (44,0,"Load Picture",A_NORMAL);
	InputStr (file,8);
	
	//opens the file, and checks for errors
	fp = fopen (file,"rb");
	if (fp==NULL)
	{
		DrawStr (5,40,"Error",A_NORMAL);
		DrawStr (80,40,"Hit any key",A_NORMAL);
		ngetchx ();
		return 1;
	}
	
	//reads the screen data from file and decompresses it
	fread (temp,1500,1,fp);
	extract (temp,vplane0);
	fread (temp,1500,1,fp);
	extract (temp,vplane1);
	fclose (fp);
	free (temp);
	
	DrawStr (5,40,"File Opened!",A_NORMAL);
	DrawStr (80,40,"Hit any key",A_NORMAL);
	ngetchx ();
	return 0;
}	

//draws cursor (to the real screen)
void DrawCursor (int x, int y,int size)
{
	//this handles the sizing of the cursor
	int x1=x + size-1;
	int y1=y + size-1;
			
	PortSet (plane0,239,127);
	DrawClipRect (MakeWinRect (x,y,x1,y1),&(SCR_RECT){0,0,239,127},A_XOR);
	PortSet (plane1,239,127);
	DrawClipRect (MakeWinRect (x,y,x1,y1),&(SCR_RECT){0,0,239,127},A_XOR);
}

//Used in function DrawPen
void DrawFillRect (int x, int y, int size, void *plane,int Attr)
{
	//more sizing junk
	int x1=x + size-1;
	int y1=y + size-1;		
						
	PortSet (plane,239,127);
	FillTriangle (x,y1,x,y,x1,y,&(SCR_RECT){0,0,239,127},Attr);
	FillTriangle (x1,y1,x1,y,x,y1,&(SCR_RECT){0,0,239,127},Attr);
}

//Draws to the screen with chosen color (virtual screen)
void DrawPen (int x, int y, int size, int color)
{		
	//this statement handles the different color choices
	switch (color)
	{
		case ERASE:
		{
			DrawFillRect (x,y,size,vplane0,A_REVERSE);
			DrawFillRect (x,y,size,vplane1,A_REVERSE);
			break;
		}
		
		case LGREY:
		{
			DrawFillRect (x,y,size,vplane0,A_REVERSE);
			DrawFillRect (x,y,size,vplane1,A_REVERSE);
			DrawFillRect (x,y,size,vplane0,A_NORMAL);
			break;
		}
		
		case DGREY:
		{
			DrawFillRect (x,y,size,vplane0,A_REVERSE);
			DrawFillRect (x,y,size,vplane1,A_REVERSE);
			DrawFillRect (x,y,size,vplane1,A_NORMAL);
			break;
		}
			
		case BLACK:
		{
			DrawFillRect (x,y,size,vplane0,A_REVERSE);
			DrawFillRect (x,y,size,vplane1,A_REVERSE);
			DrawFillRect (x,y,size,vplane0,A_NORMAL);
			DrawFillRect (x,y,size,vplane1,A_NORMAL);
			break;
		}
		
		default:
		{}
	}
	
}

//draws an ellipse (to real screen, used in animation)
void DrawEllipse (int x, int y, int horz, int vert)
{
	FontSetSys (F_4x6);
	PortSet (plane0,239,127);
	DrawClipEllipse (x,y,horz,vert,&(SCR_RECT){0,0,239,127},A_XOR);
	PortSet (plane1,239,127);
	DrawClipEllipse (x,y,horz,vert,&(SCR_RECT){0,0,239,127},A_XOR);
	
	DrawStr (10,max_y-6,"Ellipse Mode",A_NORMAL);	
}

//draws an ellipse with the chosen color (to virtual screen)
void ColorEllipse (int x, int y, int horz, int vert, int color)
{
	//handles the color choices
	switch (color)
	{
		case LGREY:
		{			
			PortSet (vplane0,239,127);
			DrawClipEllipse (x,y,horz,vert,&(SCR_RECT){0,0,239,127},A_NORMAL);
			break;
		}
		
		case DGREY:
		{
			PortSet (vplane1,239,127);
			DrawClipEllipse (x,y,horz,vert,&(SCR_RECT){0,0,239,127},A_NORMAL);
			break;
		}
		
		case BLACK:
		{
			PortSet (vplane0,239,127);
			DrawClipEllipse (x,y,horz,vert,&(SCR_RECT){0,0,239,127},A_NORMAL);
			PortSet (vplane1,239,127);
			DrawClipEllipse (x,y,horz,vert,&(SCR_RECT){0,0,239,127},A_NORMAL);
			break;
		}
		
		default:
		{}					
	}
}

//draws line (to real screen, used in animation)
void MyDrawLine (int x, int y, int x1, int y1)
{
	FontSetSys (F_4x6);
	PortSet (plane0,239,127);
	DrawClipLine (MakeWinRect (x,y,x1,y1), &(SCR_RECT){0,0,239,127},A_XOR);
	PortSet (plane1,239,127);
	DrawClipLine (MakeWinRect (x,y,x1,y1), &(SCR_RECT){0,0,239,127},A_XOR);
	
	//draw some boxes at the ends of the line (makes them easier to see)
	DrawFillRect (x-2,y-2,4,plane0,A_NORMAL);
	DrawFillRect (x-2,y-2,4,plane1,A_NORMAL);
	DrawFillRect (x1-2,y1-2,4,plane0,A_NORMAL);
	DrawFillRect (x1-2,y1-2,4,plane1,A_NORMAL);
	
	DrawStr (10,max_y-6,"Line Mode",A_NORMAL);
}

//draws line with chosen color (to virtual screen)
void ColorLine (int x, int y, int x1, int y1, int color)
{
	//handles colors
	switch (color)
	{
		case LGREY:
		{			
			PortSet (vplane0,239,127);
			DrawClipLine (MakeWinRect (x,y,x1,y1),&(SCR_RECT){0,0,239,127},
										A_NORMAL);
			break;
		}
		
		case DGREY:
		{
			PortSet (vplane1,239,127);
			DrawClipLine (MakeWinRect (x,y,x1,y1),&(SCR_RECT){0,0,239,127},
										A_NORMAL);
			break;
		}
		
		case BLACK:
		{
			PortSet (vplane0,239,127);
			DrawClipLine (MakeWinRect (x,y,x1,y1),&(SCR_RECT){0,0,239,127},
										A_NORMAL);
			PortSet (vplane1,239,127);
			DrawClipLine (MakeWinRect (x,y,x1,y1),&(SCR_RECT){0,0,239,127},
										A_NORMAL);
			break;
		}
		
		default:
		{}		
	}
}

/*this function determines what mode the program is in, and chooses the 
correct drawing function accordingly*/

void DrawChoose(int x, int y, int x1, int y1, int size,
							int color, int horz, int vert, int mode)
{
	//handles the different modes
	switch (mode)
	{
		case NONE:
		{
			PageFlip ();
			DrawCursor (x,y,size);
			break;
		}
		
		case DRAW:
		{
			DrawPen(x,y,size,color);
			PageFlip ();
			DrawCursor (x,y,size);
			break;
		}
		
		case CIRCLE:
		{
			PageFlip ();
			DrawEllipse (x,y,horz,vert);
			break;
		}
		
		case LINE:
		{
			PageFlip ();
			MyDrawLine (x,y,x1,y1);
			break;
		}
		
		default: {}
	}
}

